<?php

namespace Litecms\ActivityProject\Providers;

use Illuminate\Support\ServiceProvider;

class ActivityProjectServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'activity_project');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'activity_project');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerActivityProject();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerActivityProject()
    {
        $this->app->bind('activity_project', function($app) {
            return new ActivityProject($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('ActivityProject', 'Lavalite\ActivityProject\Facades\ActivityProject');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litecms.activity_project', function ($app) {
            return $this->app->make('Litecms\ActivityProject\ActivityProject');
        });

                // Bind ActivityProject to repository
        $this->app->bind(
            'Litecms\ActivityProject\Interfaces\ActivityProjectRepositoryInterface',
            \Litecms\ActivityProject\Repositories\Eloquent\ActivityProjectRepository::class
        );

        $this->app->register(\Litecms\ActivityProject\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Litecms\ActivityProject\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and activity_project's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/activity_project.php', 'activity_project'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeActivityProject::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litecms.activity_project'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litecms/activity_project.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/activity_project')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/activity_project')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
